// $Id: cspcm.cpp,v 1.1.1.1 2004/05/09 03:04:06 tkudou Exp $

#include <stdio.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <synthe.h>
#include <stdlib.h>

#include "cspcm.h"


CSPCM::CSPCM()
{
  pcPcmBuf = 0;
  driver_pcm = 0;
}

CSPCM::~CSPCM()
{
  ::SYT_syntheEnd(SynHandle);
  if (pcPcmBuf)
  {
    free(pcPcmBuf);
  }
  delete driver_pcm;
}

bool CSPCM::open(const char * dic_name, const char * pcm_device, int speed)
{
  int err;
  SynHandle = ::SYT_syntheInit(dic_name, STY_WAVE_KIND_MALE , STY_WAVE_DATA_16LINEAR, &err, "syntest_mmf");
  if ( SynHandle == NULL )
  {
    return false;
  }
  /*
   * PcmХåեγ
   */
  pcPcmBuf = (char *)malloc ( PCMSIZE );
  if ( pcPcmBuf == NULL )
  {
    return false;
  }
  driver_pcm = new DriverPCM;
  driver_pcm->open(pcm_device);
  ::SYT_setSpeed(SynHandle, speed);
  return true;
}

void CSPCM::close()
{
  if (driver_pcm)
  {
    delete driver_pcm;
    driver_pcm = 0;
  }
}

static bool
mute_cmd_arrived_p()
{
#define IPCIF_MSG_KEY 0xBACD0830
#define MSGMAX 8192
#define IPCIF_CMD_MUTE 61

  static int ipcif_qid = 0;
  if (!ipcif_qid)
  {
    key_t cmd_key = IPCIF_MSG_KEY;
    ipcif_qid = msgget(cmd_key, 0);
    // printf("IPCIF initialized qid:%d\n", ipcif_qid);
  }
  bool ret = false;
  if (ipcif_qid)
  {
    char buf[MSGMAX * 2];
    const long id = IPCIF_CMD_MUTE;
    if (msgrcv(ipcif_qid, buf, MSGMAX, id, IPC_NOWAIT) >= 0)
    {
      ret = true;
    }
  }
  return ret;
}

bool CSPCM::say(const char *msg, int msg_len)
{
  int iErrorInfo = ::SYT_generatePcmStart ( SynHandle, msg, msg_len );
  if ( iErrorInfo != SYERR_NOERROR )
  {
    return false;
  }
  static int say_n = 0;
  // printf("SAY:%d\n", say_n++);
  for ( ; ; )
  {
    DWORD dwGenSize;

    if (mute_cmd_arrived_p())
    {
      // printf("SAY:LOOP-STOP\n");
      driver_pcm->stop();
      // voice cancel
      ::SYT_generatePcmStop(SynHandle);
      return true;
    }
    iErrorInfo = SYT_generatePcm ( SynHandle, 
				   pcPcmBuf, 
				   PCMSIZE, 
				   &dwGenSize );
    if ( iErrorInfo == SYRET_CONTINUE || iErrorInfo == SYRET_END )
    {
      /* PCMǡpszPCM˽񤭹 */
      if ( !driver_pcm->send(pcPcmBuf, dwGenSize/2))
      {
	return false;
      }
      /* ⤦PCMǡϻĤäƤʤ */
      if ( iErrorInfo == SYRET_END )
      {
	::SYT_generatePcmStop(SynHandle);
	return true;
      }
    }
    else
    {
      return false;
    }
  }
  return true;
}

