/*
 * BRLTTY - A background process providing access to the Linux console (when in
 *          text mode) for a blind person using a refreshable braille display.
 *
 * Copyright (C) 1995-2003 by The BRLTTY Team. All rights reserved.
 *
 * BRLTTY comes with ABSOLUTELY NO WARRANTY.
 *
 * This is free software, placed under the terms of the
 * GNU General Public License, as published by the Free Software
 * Foundation.  Please see the file COPYING for details.
 *
 * Web Page: http://mielke.cc/brltty/
 *
 * This software is maintained by Dave Mielke <dave@mielke.cc>.
 */

/* Festival/speech.c - Speech library
 * For the Festival text to speech package
 * Maintained by Nikhil Nair <nn201@cus.cam.ac.uk>
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

typedef enum {
  PARM_LangDic,
  PARM_PcmDic,
  PARM_LogPath,
  PARM_PcmDevice,
  PARM_Speed,
  PARM_Test1,
  PARM_Test2
/*
  PARM_IniFile,
  PARM_SampleRate,
  PARM_AbbreviationMode,
  PARM_NumberMode,
  PARM_SynthMode,
  PARM_TextMode,
  PARM_Language,
  PARM_Voice,
  PARM_VocalTract,
  PARM_Breathiness,
  PARM_HeadSize,
  PARM_PitchBaseline,
  PARM_PitchFluctuation,
  PARM_Roughness,
  PARM_Volume
*/
} 	DriverParameter;
#define SPKPARMS "langdic", "pcmdic", "logpath", "pcmdevice", "speed", "test1", "test2"

#include "Programs/misc.h"
#include "Programs/spk.h"
#include "Programs/spk_driver.h"

#include "speech.h"		/* for speech definitions */
#include "Programs/misc.h"

#include "csdriver.h"
#include <string.h>


static void
spk_identify (void)
{
  LogPrint(LOG_NOTICE, "The Create System Japanese Language Speech Library Ver 1.1");
}

CSDriver * driver;
FILE * pcm_log_file = 0;

static int
spk_open (char **parameters)
{
  int speed;
  const char *lang_dic = parameters[PARM_LangDic];
  const char *pcm_dic = parameters[PARM_PcmDic];
  const char *log_path = parameters[PARM_LogPath];
  const char *pcm_device = parameters[PARM_PcmDevice];
  const char *pspeed = parameters[PARM_Speed];
//  const char *test2 = parameters[PARM_Test2];
  

  if (lang_dic == 0)
  {
    lang_dic = "/usr/local/synth/dic/wd2k03.dic";
  }
  if (pcm_dic == 0)
  {
    pcm_dic = "/usr/local/synth/dic/HIGH16.VSD";
  }
  if (pcm_device == 0)
  {
    pcm_device = "/dev/dsp";
  }
  if (log_path)
  {
    LogPrint(LOG_NOTICE, (char *) log_path);
    pcm_log_file = fopen(log_path,"w");
  }
  speed = 5;
  if (pspeed)
  {
    speed = atoi(pspeed);
  }
  
  driver = new CSDriver;
  driver->open(lang_dic, pcm_dic, pcm_device, speed);
  return 1;
}

static void
spk_close (void)
{
  if (pcm_log_file)
  {
    fclose(pcm_log_file);
  }
  if (driver)
  {
    delete driver;
  }
}

static void
spk_say (const unsigned char *buffer, int len)
{
  if (pcm_log_file)
  {
    int n;
    fprintf(pcm_log_file, "input [");
    for (n = 0; n < len; n++)
    {
      fputc(buffer[n], pcm_log_file);
    }
    fputc(']', pcm_log_file);
    fputc('\n', pcm_log_file);
    fflush(pcm_log_file);
  }
  
  if (buffer)
  {
    if (len)
    {
      driver->sayEuc((const char *) buffer, len);
    }
  }
}

// stop message
static void
spk_mute (void)
{
//  printf("MUTE\n");
}
