/* $Id: braille.c,v 1.1.1.1 2004/05/09 03:04:06 tkudou Exp $ */
/*
 * BRLTTY - A background process providing access to the Linux console (when in
 *          text mode) for a blind person using a refreshable braille display.
 *
 * Copyright (C) 1995-2003 by The BRLTTY Team. All rights reserved.
 *
 * BRLTTY comes with ABSOLUTELY NO WARRANTY.
 *
 * This is free software, placed under the terms of the
 * GNU General Public License, as published by the Free Software
 * Foundation.  Please see the file COPYING for details.
 *
 * Web Page: http://mielke.cc/brltty/
 *
 * This software is maintained by Dave Mielke <dave@mielke.cc>.
 */

/* KGS/braille.c - Braille display library for KGS braille displays
 * Copyright (C) 2004 by CYPAC Co., Inc. <http://www.cypac.co.jp/>
 * See the GNU Public license for details in the ../../COPYING file
 *
 */

/* Changes:
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/termios.h>
#include <string.h>

#include "Programs/misc.h"
#include "Programs/brltty.h"

/*#define BRLSTAT ST_KGSStyle*/
#include "Programs/brl_driver.h"
#include "braille.h"

#define DEBUG

static int brl_fd;		/* file descriptor for Braille display */
static struct termios oldtio;		/* old terminal settings */


#define MAX_STCELLS	5	/* hiest number of status cells */


/* This is the brltty braille mapping standard to mapping table
 * for KGS Braille Memo/Note. */
static TranslationTable outputTable;



/* Global variables */

static unsigned char *rawdata;	/* translated data to send to Braille */
static unsigned char StatusCells[MAX_STCELLS];	/* to hold status info */
static unsigned char PrevStatus[MAX_STCELLS];	/* to hold previous status */
int BrlRows	= -1;
int Cols	= -1;
int NbStCells	= -1;
static int ReWrite = 0;		/* 1 if display need to be rewritten */



/* Communication codes */

#ifdef ALVA_KEYBOARD
/* ALVA ALVA ALVA ALVA ALVA ALVA ALVA ALVA ALVA ALVA ALVA ALVA ALVA ALVA */
static char BRL_ID[] = "\033ID=";
#define DIM_BRL_ID 4
#endif /* ALVA_KEYBOARD */


/* Key values */

#ifdef ALVA_KEYBOARD
/* NB: The first 7 key values are the same as those returned by the
 * old firmware, so they can be used directly from the input stream as
 * make and break sequence already combined... not to be changed.
 */
#define KEY_PROG 	0x008	/* the PROG key */
#define KEY_HOME 	0x004	/* the HOME key */
#define KEY_CURSOR 	0x002	/* the CURSOR key */
#define KEY_UP 		0x001	/* the UP key */
#define KEY_LEFT 	0x010	/* the LEFT key */
#define KEY_RIGHT 	0x020	/* the RIGHT key */
#define KEY_DOWN 	0x040	/* the DOWN key */
#define KEY_CURSOR2 	0x080	/* the CURSOR2 key */
#define KEY_HOME2 	0x100	/* the HOME2 key */
#define KEY_PROG2 	0x200	/* the PROG2 key */
#define KEY_ROUTING	0x400	/* cursor routing key set */

/* those keys are not supposed to be combined, so their corresponding 
 * values are not bit exclusive between them.
 */
#define KEY_ROUTING_A	0x1000	/* first cur routing over status display */
#define KEY_ROUTING_B	0x2000	/* second cur routing over status disp. */
#define KEY_ROUTING_C	0x3000	/* ... */
#define KEY_ROUTING_D	0x4000
#define KEY_ROUTING_E	0x5000
#define KEY_ROUTING_F	0x6000

/* first cursor routing offset on main display (old firmware only) */
#define KEY_ROUTING_OFFSET 168

/* Index for new firmware protocol */
static int OperatingKeys[10] =
{KEY_PROG, KEY_HOME, KEY_CURSOR,
 KEY_UP, KEY_LEFT, KEY_RIGHT, KEY_DOWN,
 KEY_CURSOR2, KEY_HOME2, KEY_PROG2};
static int StatusKeys[6] =
{KEY_ROUTING_A, KEY_ROUTING_B, KEY_ROUTING_C,
 KEY_ROUTING_D, KEY_ROUTING_E, KEY_ROUTING_F};
#endif /* ALVA_KEYBOARD */

/* KGS */
#define KGS_DELAY	100 /* msec / loop */

#define KGS_TRUE	0x01
#define KGS_FALSE	0x00

#define KGS_OC_QUERY_EQUIPMENT		0x10
#define KGS_OC_QUERY_LINE_SIZE		0x11
#define KGS_OC_START_DISPLAY_MODE	0x20
#define   KGS_BLINK_NONE			0x00
#define   KGS_BLINK_SLOW			0x01
#define   KGS_BLINK_FAST			0x02
#define   KGS_BLINK_DEFAULT			KGS_BLINK_SLOW
#define KGS_OC_START_SERVER_MODE	0x21
#define KGS_OC_END_DISPLAY_MODE		0x28
#define KGS_OC_END_SERVER_MODE		0x29
#define KGS_OC_SENDING_DISPLAY_DATA	0x31
#define KGS_OC_SENDING_BRINK_CELL_INFO	0x33
#define KGS_OC_SET_BRAILLE_MODE		0x34
#define KGS_OC_QUERY_SENDING_TEXT	0x40
#define KGS_OC_START_SENDING_TEXT	0x41
#define KGS_OC_END_SENDING_TEXT		0x42
#define KGS_OC_SENDING_TEXT_LINE	0x43
#define KGS_OC_QUIT_SENDING_TEXT	0x44
#define KGS_OC_CANCEL_SENDING_TEXT	0x45
#define KGS_OC_REQUEST_TEXT_HEADER	0x48
#define KGS_OC_SENDING_TEXT_HEADER	0x49
#define KGS_OC_REQUEST_TEXT_DATA	0x4a
#define KGS_OC_REQUEST_MONITOR_INFO	0x60
#define KGS_OC_SENDING_MONITOR_INFO	0x61
#define KGS_OC_SET_CLOCK		0x81
#define KGS_OC_REQUEST_PARAMETERS	0x82
#define KGS_OC_SENDING_PARAMETERS	0x83
#define KGS_OC_GET_BANK_NUMBER		0x84
#define KGS_OC_SET_BANK_NUMBER		0x85
#define KGS_OC_SENDING_BM1_KEY_INFO	0xf0

/* byte 0: KT -> key types */
/* byte 1: KC -> key codes */
#define KGS_KT_NONE	0
#define KGS_KT_BRAILLE	1
#define KGS_KT_EDIT	2
#define   KGS_KC_ESC		0x01
#define   KGS_KC_INFO		0x02
#define   KGS_KC_BS		0x04
#define   KGS_KC_DEL		0x08
#define   KGS_KC_INS		0x10
#define   KGS_KC_CHANGE		0x20
#define   KGS_KC_OK		0x40
#define   KGS_KC_SET		0x80
#define KGS_KT_ARROW	3
#define   KGS_KC_UP		0x01
#define   KGS_KC_DOWN		0x02
#define   KGS_KC_LEFT		0x04
#define   KGS_KC_RIGHT		0x08
#define KGS_KT_CURSOR	4
#define KGS_KT_DISPLAY	6
#define   KGS_KC_BACKWARD	0x01
#define   KGS_KC_FORWARD	0x02
#define   KGS_KC_LSCROLL	0x04
#define   KGS_KC_RSCROLL	0x08
/* byte 2: shift keys */
#define   KGS_KC_LSHIFT		0x01
#define   KGS_KC_RSHIFT		0x02
#define   KGS_KC_CTRL		0x04
#define   KGS_KC_ALT		0x08
#define   KGS_KC_SELECT		0x10
#define   KGS_KC_READ		0x20
#define   KGS_KC_F2		0x40
#define   KGS_KC_F3		0x80


static void
brl_identify (void)
{
  LogPrint(LOG_NOTICE, "KGS driver for BM16/BN46X, version 0.1");
  LogPrint(LOG_INFO,
    "   Copyright (C) 2004 by Cypac Corporation <http://www.cypac.co.jp/>.");
}


#include <errno.h>

typedef unsigned char	uchar;

#ifdef DEBUG
static void
debug_log_bytes(uchar *name, uchar *data, size_t len)
{
  int i;
  char buf[1024];
  char *p = buf;
  p += sprintf(buf, "%02x", data[0]);
  for (i=1; i<len; ++i)
    p += sprintf(p, ",%02x", data[i]);
  LogPrint(LOG_INFO/*DEBUG*/, "%s(%d): %s", name, len, buf);
}
#else /* ! DEBUG */
#define debug_log_bytes(x, y, z)	/**/
#endif /* ! DEBUG */

static int
SendToBraille(uchar *data, size_t len)
     /* bool */
     /* return 1 if succeed. */
     /* return 0 if failure. */
{
  /*debug_log_bytes("SendToBraille", data, len);*/

  if (safe_write( brl_fd, data, len ) == len) return 1;

  LogPrint(LOG_ERR, "KGS/"__FILE__"(%d): SendToBraille failure. errno:%d",
	   __LINE__, errno);
  return 0;
}

static int
SendKGSCommand( uchar command, uchar subcom, size_t size, uchar *data )
     /* bool */
     /* return 1 if succeed. */
     /* return 0 if failure. */
{
  uchar buf[6] = { 0xff, 0xff, command, subcom, size & 0xff, size>>8 } ;
  if (! SendToBraille(buf, sizeof(buf)))
    return 0;
  if (size > 0)
    if (! SendToBraille(data, size))
      return 0;
  return 1;
}

static int
read_sized_data( uchar *buf, int size, int wait )
     /* bool */
     /* return read size. */
{
#if 1
  delay(wait * 1000);
#else /* ! 1 */
  fd_set rfds;
  struct timeval tv;
  int retval;
  FD_ZERO(&rfds);
  FD_SET(brl_fd, &rfds);
  tv.tv_sec = wait;
  tv.tv_usec = 0;
  retval = select(1, &rfds, NULL, NULL, &tv);
  if (! retval) return 0;
#endif /* ! 1 */
  return read(brl_fd, buf, size);
}


static int
brl_open (BrailleDisplay *brl, char **parameters, const char *dev)
{
  int ModelID = MODEL;
  struct termios newtio;	/* new terminal settings */

  brl->writeDelay = KGS_DELAY;

  {
    /* -> http://www.kgs-jpn.co.jp/dev/protocol.html OC_SENDING_DISPLAY_DATA */
    static const DotsTable dots =
      {0x80, 0x40, 0x20, 0x08, 0x04, 0x02, 0x10, 0x01};
    makeOutputTable(&dots, &outputTable);
  }

  rawdata = NULL;		/* clear pointer */

  /* Open the Braille display device for random access */
  if (!openSerialDevice(dev, &brl_fd, &oldtio)) goto failure;

  /* Set flow control and 8n1, enable reading */
  newtio.c_cflag = CS8 | CLOCAL | CREAD;

  /* Ignore bytes with parity errors and make terminal raw and dumb */
  newtio.c_iflag = IGNPAR;
  newtio.c_oflag = 0;		/* raw output */
  newtio.c_lflag = 0;		/* don't echo or generate signals */
  newtio.c_cc[VMIN] = 0;	/* set nonblocking read */
  newtio.c_cc[VTIME] = 0;

  uchar rbuf[32];
  uchar *KGSName;

  /* autodetecting KGS terminal model */
  do
    {
      /* activate new settings */
      resetSerialDevice(brl_fd, &newtio, KGS_DEFAULT_BAUDRATE);
      delay(1000);		/* delay before 2nd line drop */
      /* This "if" statement can be commented out to try autodetect once anyway */
      if (ModelID != KGS_AUTO)
	break;

      LogPrint(LOG_DEBUG, "Debug: KGS/"__FILE__"(%d): detecting...", __LINE__);

      if (! SendKGSCommand(KGS_OC_QUERY_EQUIPMENT, 0x00, 0, NULL))
	goto failure;
      {
	int r = read_sized_data(rbuf, 32, 5);
	if (r != 32)
	  {
	    LogPrint(LOG_ERR, "KGS/"__FILE__"(%d): "
		     "OC_QUERY_EQUIPMENT failure: r(%d) != 32", __LINE__, r);
	    goto failure;
	  }
      }
      debug_log_bytes("rbuf", rbuf, 32);

      if (rbuf[0] != 0x01)
	{
	  LogPrint(LOG_ERR,
		   "KGS/"__FILE__"(%d): Protocol error: Device category: %x",
		   __LINE__, rbuf[0]);
	  goto failure;
	}
      if (rbuf[1] != KGS_BM16 &&
	  rbuf[1] != KGS_BN46X)
	{
	  LogPrint(LOG_ERR, "KGS/"__FILE__"(%d): Bad device ID:%x", __LINE__,
		   rbuf[1]);
	  goto failure;
	}

      if (rbuf[4] != 1)
	LogPrint(LOG_WARNING, "KGS/"__FILE__"(%d): Warning: lines(%d) != 0",
		 __LINE__, rbuf[4]);

      if (rbuf[6] != 0)
	LogPrint(LOG_WARNING,
		 "KGS/"__FILE__"(%d): Protocol warning: rbuf[6](%d) != 0",
		 __LINE__, rbuf[6]);
      if (rbuf[7] != 0)
	LogPrint(LOG_WARNING,
		 "KGS/"__FILE__"(%d): Protocol warning: rbuf[7](%d) != 0",
		 __LINE__, rbuf[7]);

      ModelID = rbuf[1];
      BrlRows = rbuf[4];
      Cols = rbuf[5];
      NbStCells = 0;
      KGSName = rbuf+8;	/* !!! !!! !!! This string is SHIFT-JIS !!! !!! !!! */

      LogPrint(LOG_INFO,
	       /*"KGS/"__FILE__"(%d): "*/
	       "KGS Device ID: %x, V%d.%d"
	       /*", Cols,Rows: %d,%d"*/
	       ,
	       /*__LINE__,*/
	       ModelID, rbuf[2], rbuf[3]
	       /*, Cols, BrlRows*/
	       );
      /* columns and rows will be
	 print in Programs/brl.c:brailleBufferResized() */

      /* !!! This string is SHIFT-JIS !!! */
      LogPrint(LOG_INFO, "KGS Device Name: \"%s\"", KGSName);
    }
  while (ModelID == KGS_AUTO);

  /*
  LogPrint( LOG_INFO, "Detected KGS model %s: %d columns, %d status cells.",
            KGSName, Cols, NbStCells);
  */

  {
    uchar data[] = { KGS_BLINK_DEFAULT, 0x00 } ;
    if (! SendKGSCommand(KGS_OC_START_DISPLAY_MODE, 0x00, sizeof(data), data))
      goto failure;
  }
  {
    int r = read_sized_data(rbuf, 1, 5);
    if (r != 1)
      {
	LogPrint(LOG_ERR, "KGS/"__FILE__"(%d): "
		 "OC_START_DISPLAY_MODE failure: r(%d) != 1",
		 __LINE__, rbuf[0]);
	goto failure;
      }
  }
  if (rbuf[0] != KGS_TRUE) {
    LogPrint(LOG_ERR,
	     "KGS/"__FILE__"(%d): OC_START_DISPLAY_MODE failure: %x",
	     __LINE__, rbuf[0]);
    goto failure;
  }



  /* Set model params... */
  /* too many help screens, too little difference between them, so for now... */
  /* brl->helpPage = model - Models; */
  brl->x = Cols;			/* initialise size of display */
  brl->y = BrlRows;

  /* Allocate space for buffers */
  rawdata = (unsigned char *) malloc (brl->x * brl->y);
  if (!rawdata)
    {
      LogPrint( LOG_ERR, "Cannot allocate braille buffers." );
      goto failure;
    }

  ReWrite = 1;			/* To write whole display at first time */

  return 1;

failure:
  if (rawdata)
    free (rawdata);
  return 0;
}


static void
brl_close (BrailleDisplay *brl)
{
  free (rawdata);
  tcsetattr (brl_fd, TCSADRAIN, &oldtio);	/* restore terminal settings */
  close (brl_fd);
}


static int
WriteToBrlDisplay (int Start, int Len, unsigned char *Data)
{
  if (Start != 0)
    LogPrint(LOG_WARNING, "KGS/"__FILE__"(%d): not supported: Start(%d) != 0",
	     __LINE__, Start);
  char buf[Cols];
  memset(buf, 0, Cols);
  memcpy(buf, Data, Len);
  return SendKGSCommand(KGS_OC_SENDING_DISPLAY_DATA, 0x00, Cols, buf);
}

static void
brl_writeWindow (BrailleDisplay *brl)
{
  int i;
  static int Timeout = 0;

  if (ReWrite ||  ++Timeout > (REFRESH_RATE/refreshInterval))
    ReWrite = Timeout = 0;

  for (i=0 ; i<Cols ; ++i)
    rawdata[i] = outputTable[brl->buffer[i]];
  brl->writeDelay = KGS_DELAY;
  WriteToBrlDisplay(NbStCells, Cols, rawdata);
}


static void
brl_writeStatus (BrailleDisplay *brl, const unsigned char *st)
{
  /* Update status cells on braille display */
  if (memcmp (st, PrevStatus, NbStCells))	/* only if it changed */
    {
      int i;
      for (i = 0;
	   i < NbStCells;
	   StatusCells[i++] = outputTable[(PrevStatus[i] = st[i])])
	;
      WriteToBrlDisplay (0, NbStCells, StatusCells);
    }
}

#define MAX_RS_BUFF 100

static unsigned int
GetKGSKey (unsigned int *key_stat, unsigned int * key_shift, unsigned int *Pos)
{
  unsigned char buf[MAX_RS_BUFF];
  unsigned char c;
  int type = KGS_KT_NONE;
  /* buf[0] 0xff
     buf[1] 0xff
     buf[2] 0xf0 -- command KGS_OC_SENDING_BM1_KEY_INFO
     buf[3] 0x00 -- reserved
     buf[4] key_type
     buf[5] key_stat
     buf[6] key_shift
  */
  if (read (brl_fd, &c, 1) > 0)
  {
    if (c == 0xff)
    {
      read(brl_fd, buf, 1);
      if (buf[0] == 0xff)
	read(brl_fd, &buf, 1);
      read(brl_fd, buf+1, 3);
      int len = buf[2] + (buf[1] << 8);
      if (len< MAX_RS_BUFF)
	read(brl_fd, buf+4, len);
    }
//  printf("0:%02x, 1:%02x, 2:%02x, 3:%02x, 4:%02x, 5:%02x, 6:%02x, 7:%02x\n",
//	   buf[0], buf[1], buf[2], buf[3], buf[4], buf[5], buf[6],buf[7]);
    if (buf[0] == KGS_OC_SENDING_BM1_KEY_INFO)
    {
      type = buf[4];
      *key_stat = buf[5];
      *key_shift = buf[6];
      if (type == KGS_KT_CURSOR)
	*Pos = buf[5];
    }
  }
  return type;
}


static int
brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds)
{
  unsigned int type, stat, shift, pos;

  type = GetKGSKey (&stat, &shift, &pos);
  if (type == KGS_KT_NONE)
    return EOF;

  ReWrite = 1;
  LogPrint(LOG_INFO/*DEBUG*/,
	   "KGS/"__FILE__"(%d): type(%x) stat(%x) shift(%x) pos(%x)",
	   __LINE__, type, stat, shift, pos);
  switch (type) {
  case KGS_KT_BRAILLE:
    LogPrint(LOG_WARNING, "KGS/"__FILE__"(%d): "
	     "Sorry, braille keys are not supported yet.", __LINE__);
    return EOF;
  case KGS_KT_EDIT:
    switch (stat) {
    case KGS_KC_ESC:	return	CMD_MUTE_VOICE;
    case KGS_KC_INFO:	return	CMD_SAY_LINE;
    case KGS_KC_BS:
      if (shift & KGS_KC_LSHIFT)
	return			CMD_TOP;
      return			CMD_BOT;
    case KGS_KC_DEL:	return	CMD_NOOP;
    case KGS_KC_INS:	return	CMD_NOOP;
    case KGS_KC_CHANGE:	return	CMD_NOOP;
    case KGS_KC_OK:	return	CMD_NOOP;
    case KGS_KC_SET:	return	CMD_NOOP;
    default:
      LogPrint(LOG_ERR, "KGS/"__FILE__"(%d): Bad keycode: %x", __LINE__, stat);
      return			CMD_NOOP;
    }
  case KGS_KT_ARROW:
    switch (stat) {
    case KGS_KC_UP:	return	CMD_LNUP;
    case KGS_KC_DOWN:	return	CMD_LNDN;
    case KGS_KC_LEFT:	return	CMD_CHRLT;
    case KGS_KC_RIGHT:	return	CMD_CHRRT;
    default:
      LogPrint(LOG_ERR, "KGS/"__FILE__"(%d): Bad keycode: %x", __LINE__, stat);
      return			CMD_NOOP;
    }
  case KGS_KT_CURSOR:
    return CR_ROUTE + pos;
  case KGS_KT_DISPLAY:
    switch (stat) {
    case KGS_KC_BACKWARD: return CMD_WINUP;
    case KGS_KC_FORWARD:  return CMD_WINDN;
    case KGS_KC_LSCROLL: return	CMD_FWINLT;
    case KGS_KC_RSCROLL: return	CMD_FWINRT;
    default:
      LogPrint(LOG_ERR, "KGS/"__FILE__"(%d): Bad keycode: %x", __LINE__, stat);
      return			CMD_NOOP;
    }
  default:
    LogPrint(LOG_ERR, "KGS/"__FILE__"(%d): Bad keytype: %u", __LINE__, type);
  }
  return EOF;
}

#ifdef ALVA_KEYBOARD
static int
GetABTKey (unsigned int *Keys, unsigned int *Pos)
{
  unsigned char c;
  static int KeyGroup = 0;
  static int id_l = 0;

  while (read (brl_fd, &c, 1) > 0)
    {

#if ! ABT3_OLD_FIRMWARE

      switch (KeyGroup)
	{
	case 0x71:	/* Operating keys and Status keys of Touch Cursor */
	  /* make keys */
	  if (c <= 0x09)
	    *Keys |= OperatingKeys[c];
	  else if ((c >= 0x20) && (c <= 0x25))
	    *Keys |= StatusKeys[c - 0x20];
	  /* break keys */
	  else if ((c >= 0x80) && (c <= 0x89))
	    *Keys &= ~OperatingKeys[c - 0x80];
	  else if ((c >= 0xA0) && (c <= 0xA5))
	    *Keys &= ~StatusKeys[c - 0xA0];
	  else
	    *Keys = 0;
	  KeyGroup = 0;
	  return (1);

	case 0x72:		/* Display keys of Touch Cursor */
	  /* make keys */
	  if (c <= 0x5F)
	    {			/* make */
	      *Pos = c;
	      *Keys |= KEY_ROUTING;
	    }
	  /* break keys */
	  else
	    *Keys &= ~KEY_ROUTING;
	  KeyGroup = 0;
	  return (1);

	default:
	  if ((c == 0x71) || (c == 0x72))
	    {
	      /* Key group selection */
	      KeyGroup = c;
	      id_l = 0;
	    }
	  else if (c == BRL_ID[id_l])
	    {
	      id_l++;
	      if (id_l >= DIM_BRL_ID)
		{
		  /* The terminal has been turned off and back on.  To be
		   * sure we arrange for the driver to restart so
		   * model probing, etc. will take place.
		   */
		  id_l = 0;
		  return -1;
		}
	    }
	  else
	    {
	      /* Probably garbage came on the line.
	       * Let's reset the keys and state.
	       */
	      *Keys = 0;
	      ReWrite = 1;
	      return 0;
	    }
	  break;
	}

#else /* ABT3_OLD_FIRMWARE */

      if ((c >= (KEY_ROUTING_OFFSET + Cols)) &&
	  (c <  (KEY_ROUTING_OFFSET + Cols + 6)))
	{
	  /* make for Status keys of Touch Cursor */
	  *Keys |= StatusKeys[c - (KEY_ROUTING_OFFSET + Cols)];
	}
      else if (c >= KEY_ROUTING_OFFSET)
	{
	  /* make for display keys of Touch cursor */
	  *Pos = c - KEY_ROUTING_OFFSET;
	  *Keys |= KEY_ROUTING;
	}
      else
	*Keys = c;		/* check comments where KEY_xxxx are defined */
      return (1);

#endif /* ! ABT3_OLD_FIRMWARE */

    }
  return (0);
}



static int
brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds)
{
  int ProcessKey, res = EOF;
  static unsigned int RoutingPos = 0;
  static unsigned int CurrentKeys = 0, LastKeys = 0, ReleasedKeys = 0;
  static int Typematic = 0, KeyDelay = 0, KeyRepeat = 0;

  if (!(ProcessKey = GetABTKey (&CurrentKeys, &RoutingPos)))
    {
      if (Typematic)
	{
	  /* a key is being held */
	  if (KeyDelay < TYPEMATIC_DELAY)
	    KeyDelay++;
	  else if (KeyRepeat < TYPEMATIC_REPEAT)
	    KeyRepeat++;
	  else
	    {
	      /* It's time to issue the command again */
	      CurrentKeys = LastKeys;
	      LastKeys = 0;
	      KeyRepeat = 0;
	      ProcessKey = 1;
	    }
	}
    }
  else
    {
      /* A new key is being pressed/released, we clear the typematic counters */
      Typematic = KeyDelay = KeyRepeat = 0;
    }

  if (ProcessKey < 0)
    {
      /* Oops... seems we should restart from scratch... */
      RoutingPos = 0;
      CurrentKeys = LastKeys = ReleasedKeys = 0;
      return CMD_RESTARTBRL;
    }
  else if (ProcessKey > 0)
    {
      if (CurrentKeys > LastKeys)
	{
	  /* These are the keys that should be processed when pressed */
	  LastKeys = CurrentKeys;	/* we keep it until it is released */
	  ReleasedKeys = 0;
	  switch (CurrentKeys)
	    {
	    case KEY_HOME | KEY_UP:
	      res = CMD_TOP;
	      break;
	    case KEY_HOME | KEY_DOWN:
	      res = CMD_BOT;
	      break;
	    case KEY_UP:
	      res = CMD_LNUP;
	      Typematic = 1;
	      break;
	    case KEY_CURSOR | KEY_UP:
	      res = CMD_ATTRUP;
	      break;
	    case KEY_DOWN:
	      res = CMD_LNDN;
	      Typematic = 1;
	      break;
	    case KEY_CURSOR | KEY_DOWN:
	      res = CMD_ATTRDN;
	      break;
	    case KEY_LEFT:
	      res = CMD_FWINLT;
	      break;
	    case KEY_HOME | KEY_LEFT:
	      res = CMD_LNBEG;
	      break;
	    case KEY_CURSOR | KEY_LEFT:
	      res = CMD_HWINLT;
	      break;
	    case KEY_PROG | KEY_LEFT:
	      res = CMD_CHRLT;
	      Typematic = 1;
	      break;
	    case KEY_RIGHT:
	      res = CMD_FWINRT;
	      break;
	    case KEY_HOME | KEY_RIGHT:
	      res = CMD_LNEND;
	      break;
	    case KEY_PROG | KEY_RIGHT:
	      res = CMD_CHRRT;
	      Typematic = 1;
	      break;
	    case KEY_CURSOR | KEY_RIGHT:
	      res = CMD_HWINRT;
	      break;
	    case KEY_HOME | KEY_CURSOR | KEY_UP:
	      res = CMD_PRDIFLN;
	      break;
	    case KEY_HOME | KEY_CURSOR | KEY_DOWN:
	      res = CMD_NXDIFLN;
	      break;
	    case KEY_HOME | KEY_CURSOR | KEY_LEFT:
	      res = CMD_MUTE;
	      break;
	    case KEY_HOME | KEY_CURSOR | KEY_RIGHT:
	      res = CMD_SAY_LINE;
	      break;
	    case KEY_PROG | KEY_DOWN:
	      res = CMD_FREEZE;
	      break;
	    case KEY_PROG | KEY_UP:
	      res = CMD_INFO;
	      break;
	    case KEY_PROG | KEY_CURSOR | KEY_LEFT:
	      res = CMD_BACK;
	      break;
	    case KEY_ROUTING_A:
	      res = CMD_CAPBLINK;
	      break;
	    case KEY_ROUTING_B:
	      res = CMD_CSRVIS;
	      break;
	    case KEY_ROUTING_C:
	      res = CMD_CSRBLINK;
	      break;
	    case KEY_CURSOR | KEY_ROUTING_A:
	      res = CMD_SIXDOTS;
	      break;
	    case KEY_CURSOR | KEY_ROUTING_B:
	      res = CMD_CSRSIZE;
	      break;
	    case KEY_CURSOR | KEY_ROUTING_C:
	      res = CMD_SLIDEWIN;
	      break;
	    case KEY_PROG | KEY_HOME | KEY_UP:
	      res = CMD_SPKHOME;
	      break;
	    case KEY_PROG | KEY_HOME | KEY_LEFT:
	      res = CMD_RESTARTSPEECH;
	      break;
	    case KEY_PROG | KEY_HOME | KEY_RIGHT:
	      res = CMD_SAY_BELOW;
	      break;
	    case KEY_ROUTING:
	      /* normal Cursor routing keys */
	      res = CR_ROUTE + RoutingPos;
	      break;
	    case KEY_PROG | KEY_ROUTING:
	      /* marking beginning of block */
	      res = CR_CUTBEGIN + RoutingPos;
	      break;
	    case KEY_HOME | KEY_ROUTING:
	      /* marking end of block */
	      res = CR_CUTRECT + RoutingPos;
	      break;
	    case KEY_PROG | KEY_HOME | KEY_DOWN:
	      res = CMD_PASTE;
	      break;
	    case KEY_PROG | KEY_HOME | KEY_ROUTING:
	      /* attribute for pointed character */
	      res = CR_DESCCHAR + RoutingPos;
	      break;
	    }
	}
      else
	{
	  /* These are the keys that should be processed when released */
	  if (!ReleasedKeys)
	    {
	      ReleasedKeys = LastKeys;
	      switch (ReleasedKeys)
		{
		case KEY_HOME:
		  res = CMD_TOP_LEFT;
		  break;
		case KEY_CURSOR:
		  res = CMD_HOME;
		  ReWrite = 1;	/* force rewrite of whole display */
		  break;
		case KEY_PROG:
		  res = CMD_HELP;
		  /*res = CMD_SAY_LINE;*/
		  break;
		case KEY_PROG | KEY_HOME:
		  res = CMD_DISPMD;
		  break;
		case KEY_HOME | KEY_CURSOR:
		  res = CMD_CSRTRK;
		  break;
		case KEY_PROG | KEY_CURSOR:
		  res = CMD_PREFMENU;
		  break;
		}
	    }
	  LastKeys = CurrentKeys;
	  if (!CurrentKeys)
	    ReleasedKeys = 0;
	}
    }
  return (res);
}
#endif /* ALVA_KEYBOARD */
