/* $Id: ipcif.h,v 1.1.1.1 2004/05/09 03:04:09 tkudou Exp $ */

/*!
 * \file
 * \brief This file defines the types, defines and functions of IPCIF.
 */

#ifndef INCLUDE_IPCIF_H
#define INCLUDE_IPCIF_H

/*!
 * \def WITH_IPCIF
 * \brief IPCIF ѤˤϤΥޥƤʤФʤޤ
 */
#ifdef WITH_IPCIF

#define NOT_USE_IPCIF_SEM_CTRL

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*!
 * bool type
 */
typedef enum _IPCIF_Bool
{
  IPCIF_False = 0,
  IPCIF_True = 1,
  IPCIF_Failure = 0,
  IPCIF_Success = 1,
} IPCIF_Bool;

/*!
 * IPCIF νԤޤ
 * This function initializes the IPCIF.
 *
 * \retval IPCIF_Success 
 * \retval IPCIF_Failure 
 */
IPCIF_Bool ipcif_initialize();

/*!
 * IPCIF νλԤޤ
 * This function uninitializes the IPCIF.
 *
 * \param remove_resource_p IPC ֥Ȥ IPCIF_True ˻ꤷޤ
 * \retval IPCIF_Success 
 * \retval IPCIF_Failure 
 */
IPCIF_Bool ipcif_finalize(IPCIF_Bool remove_resource_p);

/* ------------------------------------------------------------------------
 * The IPC message interfaces
 */

/*! IPC message key */
#define IPCIF_MSG_KEY 0xBACD0830

/*!
 * IPCIF ޥɤ긵򼨤ޤ
 */
typedef enum _IPCIF_Sender
{
  IPCIF_S_Unknown = 0,		/*!< unknown sender */
  IPCIF_S_Screen = 0x01,	/*!< from the screen process */
  IPCIF_S_Im = 0x02,		/*!< from the imput method */
  IPCIF_S_Any = 0xff,		/*!< any sender */
} IPCIF_Sender;

/*!
 * IPC ̤ǡ۲å
 */
typedef struct _IPCIF_Msg
{
  /*! ޥֹ
   * 0 Υޥֹ IPC Message ¤ˤѤǤʤΤդƤ
   */
  long cmd_no;

  /*! 긵 ץꥱ ID */
  IPCIF_Sender sender;

  /*! ޥɥǡĹ */
  int cmd_data_length;

  /*! ޥɥǡ */
  char cmd_data[0];
} IPCIF_Msg;


/*!
 * ޥɥǡȤưĹĹޤ
 * Ĺ礭ޥɥǡΥåϰޤ
 */
int ipcif_max_data_length();

/*!
 * IPCIF_Msg Υ֥Ȥޤ
 * \param cmd_no ޥֹ
 * \param sender 򼨤 ID
 * \param cmd_data_length ޥɥǡĹ
 * \param cmd_data ޥɥǡ
 *
 * \return 줿 IPCIF_Msg Υ, ꤬­ʤ䡢
 * åΥ¥(ipcif_max_data_length)ۤ
 * ˤ 0 ֤ޤ
 */
IPCIF_Msg* ipcif_initMsg(long cmd_no, IPCIF_Sender sender, int cmd_data_length, char* cmd_data);

/*!
 * IPCIF_Msg Υ֥Ȥޤ
 */
void ipcif_deleteMsg(IPCIF_Msg* msg);

/*!
 * IPCIF ̤ƥåޤ
 * This function sends a message though the IPCIF.
 *
 * \param msg 륳ޥɤåȤ줿¤ΤؤΥݥ
 * \retval IPCIF_Success 
 * \retval IPCIF_Failure 
 */ 
IPCIF_Bool ipcif_sendMsg(IPCIF_Msg* msg);

/*!
 * IPCIF ̤Ϳ줿åɤ߹ߤޤ
 * This function reads a message given by the IPCIF.
 *
 * \return ɤ߹ޤ줿å֤ޤå̵ˤ 0 ֤ޤ
 */
IPCIF_Msg* ipcif_readMsg();

/*!
 * IPCIF ̤Ϳ줿ΥޥֹΥåɤ߹ߤޤ
 * This function reads a message given by the IPCIF.
 *
 * \param cmd_no ɤ߹ॳޥֹ
 * \return ɤ߹ޤ줿å֤ޤå̵ˤ 0 ֤ޤ
 */
IPCIF_Msg* ipcif_readMsgWithCmdNo(long cmd_no);

/* ------------------------------------------------------------------------
 * The IPCIF shared memory interfaces
 */

/*! max screen X dimension */
#define IPCIF_SHM_DIM_X_MAX 150

/*! max screen Y dimension
 *
 * Linux Ǥϥå塼ΥǥեȤκ祵 16384 ХȤǤ
 * IPCIF_SHM_DIM_X_MAX  IPCIF_SHM_DIM_Y_MAX ݤ 15000
 * ХȤˤʤޤ 1 Ԥ 8 ХȤȤåΥС
 * إåɤθȡ1ʬɤ߾夲ޥɤ򥭥塼˳Ǽˤ
 * 15800 ХȤɬפǤ
 * ϥ塼ξ֤ʤС1ʬɤ߾夲ޥɤ֥å
 * 줺˥塼˳ǼǤ뤳Ȥ̣ޤ
 * IPCIF_SHM_DIM_X_MAX  IPCIF_SHM_DIM_Y_MAX ͤ䤹ˤ,
 * ξѤäƤޤǽθƤ
 *
 * å塼ΥǥեȤκ祵ϥƥॳˤäѹ
 * ȤǽǤroot ¤ɬפˤʤޤbrltty ϰ̥桼
 * ǤưΤǡ¤դʧɬפޤ
 */
#define IPCIF_SHM_DIM_Y_MAX 100

/*!
 * IPC Shared memory header
 */
typedef struct
{
  /*! screen x dimension */
  int dim_x;

  /*! screen y dimension */
  int dim_y;

  /*! cursor x position */
  int cursor_x;

  /*! cursor y position */
  int cursor_y;

  /*! reserved */
  int reserved[100];

  /*! screen image */
  char img[IPCIF_SHM_DIM_Y_MAX][IPCIF_SHM_DIM_X_MAX];
  
  /*! screen attributes */
  char attr[IPCIF_SHM_DIM_Y_MAX][IPCIF_SHM_DIM_X_MAX];
} IPCIF_Shm;

#define IPCIF_SHM_KEY 0xBACD0831

/*! IPC shared memory size */
#define IPCIF_SHM_SIZE sizeof(IPCIF_Shm)

/*!
 * ͭΥɥ쥹ޤ
 * \return ͭΥɥ쥹
 */
IPCIF_Shm* ipcif_getShm();

/* ------------------------------------------------------------------------
 * The IPCIF semaphore interface
 */

#define IPCIF_SEM_KEY 0xBACD0832

/*!
 * ͭ˽񤭹ि˥åޤ
 * δؿƤӽФȥåǤޤǡץ
 * ֥å󥰤ޤ
 * Ūˤ IPC ΥޥեȤäƼƤޤ
 */
void ipcif_lockSemaphore();

/*!
 * ͭؤν񤭹ߤλ뤿åޤ
 * Ūˤ IPC ΥޥեȤäƼƤޤ
 *
 * ipcif_lockSemaphore() Ȥä,å˥ʥ뤬
 * ȯˤ,ʥϥɥǤδؿȤä
 * ʥʤȡIPCIF ȤäƤ¾Υץ
 * åǤʤʤäƤޤΤǡդƤ
 */
void ipcif_unlockSemaphore();

/*!
 * ΥץåƤ뤫ɤĴ٤ޤ
 *
 * \retval IPCIF_True åƤޤ
 * \retval IPCIF_False åƤޤ
 */
IPCIF_Bool ipcif_hasLockP();

#ifdef __cplusplus
}; /* end of extern "C" */
#endif /* __cplusplus */

#endif /* WITH_IPCIF */

#endif /* INCLUDE_IPCIF_H */
