/* $Id: ipcifcmd.h,v 1.1.1.1 2004/05/09 03:04:09 tkudou Exp $ */ 

#ifndef INCLUDE_IPCIFCMD_H
#define INCLUDE_IPCIFCMD_H

#include "ipcif.h"

#ifdef WITH_IPCIF

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*!
 * IPCIF commands.
 * This definition should be coresponded and should be expanded from the
 * DriverCommand enum of 'brldefs.h'.
 */
typedef enum _IPCIF_Cmd
{
  IPCIF_CMD_NOOP = 0 /*!< do nothing, You can't use this command through IPCIF. */,

  /* vertical motion */
  IPCIF_CMD_LNUP /*!< go up one line */,
  IPCIF_CMD_LNDN /*!< go down one line */,
  IPCIF_CMD_WINUP /*!< go up several lines */,
  IPCIF_CMD_WINDN /*!< go down several lines */,
  IPCIF_CMD_PRDIFLN /*!< go up to line with different content */,
  IPCIF_CMD_NXDIFLN /*!< go down to line with different content */,
  IPCIF_CMD_ATTRUP /*!< go up to line with different attributes */,
  IPCIF_CMD_ATTRDN /*!< go down to line with different attributes */,
  IPCIF_CMD_TOP /*!< go to top line */,
  IPCIF_CMD_BOT /*!< go to bottom line */,
  IPCIF_CMD_TOP_LEFT /*!< go to top-left corner */,
  IPCIF_CMD_BOT_LEFT /*!< go to bottom-left corner */,
  IPCIF_CMD_PRPGRPH /*!< go to last line of previous paragraph */,
  IPCIF_CMD_NXPGRPH /*!< go to first line of next paragraph */,
  IPCIF_CMD_PRPROMPT /*!< go to previous command prompt */,
  IPCIF_CMD_NXPROMPT /*!< go to next command prompt */,
  IPCIF_CMD_PRSEARCH /*!< search backward for content of cut buffer */,
  IPCIF_CMD_NXSEARCH /*!< search forward for content of cut buffer */,
  
  /* horizontal motion */
  IPCIF_CMD_CHRLT /*!< go left one character */,
  IPCIF_CMD_CHRRT /*!< go right one character */,
  IPCIF_CMD_HWINLT /*!< go left half a window */,
  IPCIF_CMD_HWINRT /*!< go right half a window */,
  IPCIF_CMD_FWINLT /*!< go left one window */,
  IPCIF_CMD_FWINRT /*!< go right one window */,
  IPCIF_CMD_FWINLTSKIP /*!< go left to non-blank window */,
  IPCIF_CMD_FWINRTSKIP /*!< go right to non-blank window */,
  IPCIF_CMD_LNBEG /*!< go to beginning of line */,
  IPCIF_CMD_LNEND /*!< go to end of line */,
  
  /* implicit motion */
  IPCIF_CMD_HOME /*!< go to cursor */,
  IPCIF_CMD_BACK /*!< return to destination of most recent explicit motion */,
  
  /* feature activation and deactivation */
  IPCIF_CMD_FREEZE /*!< toggle screen frozen/unfrozen */,
  IPCIF_CMD_DISPMD /*!< toggle display attributes/text */,
  IPCIF_CMD_SIXDOTS /*!< toggle text style 6-dot/8-dot */,
  IPCIF_CMD_SLIDEWIN /*!< toggle sliding window on/off */,
  IPCIF_CMD_SKPIDLNS /*!< toggle skipping of identical lines on/off */,
  IPCIF_CMD_SKPBLNKWINS /*!< toggle skipping of blank windows on/off */,
  IPCIF_CMD_CSRVIS /*!< toggle cursor visibility on/off */,
  IPCIF_CMD_CSRHIDE /*!< toggle hide of cursor on/off */,
  IPCIF_CMD_CSRTRK /*!< toggle cursor tracking on/off */,
  IPCIF_CMD_CSRSIZE /*!< toggle cursor style block/underline */,
  IPCIF_CMD_CSRBLINK /*!< toggle cursor blinking on/off */,
  IPCIF_CMD_ATTRVIS /*!< toggle attribute underlining on/off */,
  IPCIF_CMD_ATTRBLINK /*!< toggle attribute blinking on/off */,
  IPCIF_CMD_CAPBLINK /*!< toggle capital letter blinking on/off */,
  IPCIF_CMD_TUNES /*!< toggle alert tunes on/off */,
 
  /* mode selection */
  IPCIF_CMD_HELP /*!< toggle help mode on/off */,
  IPCIF_CMD_INFO /*!< toggle info mode on/off */,
  IPCIF_CMD_LEARN /*!< enter command learn mode */,
  
  /* preference setting */
  IPCIF_CMD_PREFMENU /*!< present preferences menu */,
  IPCIF_CMD_PREFSAVE /*!< save current preferences */,
  IPCIF_CMD_PREFLOAD /*!< restore saved preferences */,
  
  /* menu navigation */
  IPCIF_CMD_MENU_FIRST_ITEM /*!< go to first item in menu */,
  IPCIF_CMD_MENU_LAST_ITEM /*!< go to last item in menu */,
  IPCIF_CMD_MENU_PREV_ITEM /*!< go to previous item in menu */,
  IPCIF_CMD_MENU_NEXT_ITEM /*!< go to next item in menu */,
  IPCIF_CMD_MENU_PREV_SETTING /*!< change value of current item in menu to previous choice */,
  IPCIF_CMD_MENU_NEXT_SETTING /*!< change value of current item in menu to next choice */,
 
  /* speech */
  IPCIF_CMD_SAY_LINE /*!< speak current line */,
  IPCIF_CMD_SAY_ABOVE /*!< speak from top through current line */,
  IPCIF_CMD_SAY_BELOW /*!< speak from current through bottom line */,
  IPCIF_CMD_MUTE /*!< stop speaking immediately */,
  IPCIF_CMD_SPKHOME /*!< go to current/last speech position */,
  
  /* virtual terminal switching */
  IPCIF_CMD_SWITCHVT_PREV /*!< switch to previous virtual terminal */,
  IPCIF_CMD_SWITCHVT_NEXT /*!< switch to next virtual terminal */,
  
  /* miscellaneous */
  IPCIF_CMD_CSRJMP_VERT /*!< vertically route cursor to top line of window */,
  IPCIF_CMD_PASTE /*!< insert cut buffer at cursor */,
  IPCIF_CMD_RESTARTBRL /*!< reinitialize braille driver */,
  IPCIF_CMD_RESTARTSPEECH /*!< reinitialize speech driver */,

  /* IPCIF extensions */

  /*!
   * \ja ΥޥɤͿ줿ʸɤ߾夲ޤ\ja_end
   * \en A given string is voiced with this command.\en_end
   */
  IPCIF_CMD_VOICE,

  /*!
   * \ja ΥޥɤͿ줿ʸܺɤߤޤ\ja_end
   * \en A given string is voiced detailedly with this command.\en_end
   */
  IPCIF_CMD_DETAIL_VOICE,

  /*!
   * \ja Υޥɤϥ֤ΰɤ߾夲ޤ\ja_end
   * \en A character of the cursor position is voiced with this command.\en_end
   */
  IPCIF_CMD_VOICE_CHAR_OF_CURPOS,
  
  /*!
   * \ja Υޥɤϥ֤ΰܺɤߤޤ\ja_end
   * \en A character of the cursor position is voiced detailedly with this command.\en_end
   */
  IPCIF_CMD_DETAIL_VOICE_CHAR_OF_CURPOS,

  /*!
   * \ja Υޥɤϥ֤ޤɤ߾夲ޤ\ja_end
   * \en The characters from the cursor position to the end of the line is voiced with this command.\en_end
   */
  IPCIF_CMD_VOICE_LINE_FROM_CURPOS,

  /*!
   * \ja Υޥɤϥ֤ԲǤ뤫ɤ߾夲ޤ\ja_end
   * \en The number of the line and the number of the column of the 
   * cursor position is voiced with this command.\en_end
   */
  IPCIF_CMD_VOICE_LINE_COL_POS,

  /*!
   * \ja ΥޥɤϺͽǤ\ja_end
   * \en This command is planded to be removed.\en_end
   */
  IPCIF_CMD_MUTE_VOICE,
  IPCIF_CMD_MUTE_DETAIL,
} IPCIF_Cmd;

IPCIF_Bool ipcif_sendCmdWithData(IPCIF_Sender sender, IPCIF_Cmd cmd, int data_length, char* data);
IPCIF_Bool ipcif_sendCmd(IPCIF_Sender sender, IPCIF_Cmd cmd);
IPCIF_Bool ipcif_sendCmdVoice(IPCIF_Sender sender, char* str, int length);
IPCIF_Bool ipcif_sendCmdDetailVoice(IPCIF_Sender sender, char* str, int length);

/*!
 * \ja
 * Υޥɤ,IPCΥ塼ˤ벻ɤ߾夲ޥɤ˴
 * IPCIF_CMD_MUTE ޥɤޤ
 * \ja_end
 * \en
 * The voice commands stored on the IPC message queue is 
 * canceled and IPCIF_CMD_MUTE command is sent with this command.
 * \en_end
 */
IPCIF_Bool ipcif_sendCmdStopVoice(IPCIF_Sender sender);

#ifdef __cplusplus
};
#endif /* __cplusplus */

#endif /* WITH_IPCIF */

#endif /* INCLUDE_IPCIFCMD_H */
