/* $Id: test.h,v 1.1.1.1 2004/05/09 03:04:09 tkudou Exp $ */

/*\file
 * \ja
 * $B$3$N%U%!%$%k$O(BC$B8@8lMQ$N4J0W%F%9%H%9%$!<%H$rDs6!$7$^$9!#(B
 * \ja_end
 * \en 
 * This file supplies a easy testing suite for the C language.
 * \en_end
 */

#ifndef INCLUDE_TEST_H
#define INCLUDE_TEST_H

#include <stdio.h>

/*!
 * \ja
 * $B$3$N4X?t$O%F%9%H%9%$!<%H$r=i4|2=$7$^$9!#(B
 * \ja_end
 * \en
 * This function initialize this testing suite.
 * \en_end
 */
void test_initialize();

/*!
 * \ja
 * $B$3$N4X?t$O%F%9%H%9%$!<%H$N(B log $B%U%!%$%k$r;XDj$7$^$9!#(B
 * \param file $B%m%0%U%!%$%k$N(B FILE $B%]%$%s%?!#(B
 * \ja_end
 * \en
 * This function specifies the log file of this testing suite.
 * \param file, FILE pointer to the log file.
 * \en_end
 */
void test_set_log(FILE* file);

/*!
 * \ja
 * $B$3$N4X?t$O7Y9p%m%0$r=PNO$7$^$9!#(B
 * \param file $B%=!<%9%3!<%I$N%U%!%$%kL>!#DL>o(B __FILE__ $B%^%/%m$NCM$rEO$7$^$9!#(B
 * \param line $B%=!<%9%3!<%I$N9THV9f!#DL>o(B __LINE__ $B%^%/%m$NCM$rEO$7$^$9!#(B
 * \param cond_str $B>r7oJ8$r5-:\$7$?J8;zNs(B
 * \param result $B>r7oJ8$N<B9T7k2L$r<($9J8;zNs(B
 * \ja_end
 * \en
 * This function outputs a log of assertion.
 * \param file, the filename of the source code. Usually the value of __FILE__ macro is passed.
 * \param line, the line number of the source code. Usually the value of __LINE__ macro is passed.
 * \param cond_str, a string which describes the conditional sentence.
 * \param result, a result string of the conditional sentence.
 * \en_end
 */
void test_assert(char* file, unsigned int line, char* cond_str, char* result);

/*!
 * \ja
 * $B$3$N4X?t$O7Y9p$NH/@8?t$rJV$7$^$9!#(B
 * \ja_end
 * \en
 * This function returns the number of the assertion.
 * \en_end
 */
int test_get_assert_number();

/*!
 * \ja
 * $B$3$N4X?t$O%F%9%H$N<B9T7k2L$rI=<($7$^$9!#(B
 * \ja_end
 * \en
 * This function displays the retult of the testing.
 * \en_end
 */
void test_show_result();

#ifdef TEST_HOLD_IMP

static FILE* test_log = 0;
static int assert_number = 0;
static int test_number = 0;

void
test_initialize()
{
  test_log = stdout;
}

void
test_set_log(FILE* file)
{
  test_log = file;
}

void
test_assert(char* file, unsigned int line, char* cond_str, char* result)
{
  assert_number++;
  fprintf(test_log, "%s:%d ", file, line);
  fprintf(test_log, "[%s] => %s\n", cond_str, result);
  fflush(test_log);
}

int
test_get_assert_number()
{
  return assert_number;
}

void test_show_result()
{
  fprintf(test_log, "%d failures in %d unit tests\n", assert_number, test_number);
}

#endif /* TEST_HOLD_IMP */

/*!
 * \ja
 * SHOULD_BE_TRUE$B%^%/%m$O>r7o$r0z?t$H$7$FM?$($^$9!#(B
 * $B>r7o$,??$N>l9g$O!"%F%9%H%1!<%9$K9g3J$7$?$H8+$J$5$l$^$9!#(B
 * $B>r7o$,56$N>l9g$O!"%F%9%H%1!<%9$K<:GT$7$?$H8+$J$5$l$^$9!#(B
 * \ja_end
 * \en
 * SHOULD_BE_TRUE macro takes a condition as an argument.
 * If the condition is true, the test case is considered to be passed.
 * If the condition is false, the test case is considered to be failure.
 * \en_end
 */ 
#define SHOULD_BE_TRUE(cond) \
test_number++;\
if (!(cond))\
{\
  test_assert(__FILE__, __LINE__, #cond , "false");\
}

/*!
 * SHOULD_BE_FALSE$B%^%/%m$O>r7o$r0z?t$H$7$FM?$($^$9!#(B
 * $B>r7o$,56$N>l9g$O!"%F%9%H%1!<%9$K9g3J$7$?$H8+$J$5$l$^$9!#(B
 * $B>r7o$,??$N>l9g$O!"%F%9%H%1!<%9$K<:GT$7$?$H8+$J$5$l$^$9!#(B
 */ 
#define SHOULD_BE_FALSE(cond) \
test_number++;\
if ((cond))\
{\
  test_assert(__FILE__, __LINE__, #cond , "true");\
}

/*!
 * \ja
 * SHOULD_NOT_BE_NULL$B%^%/%m$O%]%$%s%?$r0z?t$H$7$FM?$($^$9!#(B
 * $B%]%$%s%?$,(BNULL$B$G$J$$>l9g$O!"%F%9%H%1!<%9$K9g3J$7$?$H8+$J$5$l$^$9!#(B
 * $B%]%$%s%?$,(BNULL$B$K$J$C$?>l9g$K$O!"%F%9%H%1!<%9$K<:GT$7$?$H8+$J$5$l$^$9!#(B
 * \ja_end
 * \us
 * SHOULD_NOT_BE_NULL macro takes a pointer as an argument.
 * If the pointer is NULL, the test case is considered to be passed.
 * If the pointer isn't NULL, the test case is considered to be failure.
 * \us_end
 */
#define SHOULD_NOT_BE_NULL(cond) \
test_number++;\
if (!(cond))\
{\
  test_assert(__FILE__, __LINE__, #cond , "null");\
}

/*!
 * SHOULD_BE_NULL$B%^%/%m$O%]%$%s%?$r0z?t$H$7$FM?$($^$9!#(B
 * $B%]%$%s%?$,(BNULL$B$N>l9g$K$O!"%F%9%H%1!<%9$K9g3J$7$?$H8+$J$5$l$^$9!#(B
 * $B%]%$%s%?$,(BNULL$B$G$J$$>l9g$K$O!"%F%9%H%1!<%9$K<:GT$7$?$H8+$J$5$l$^$9!#(B
 */
#define SHOULD_BE_NULL(cond) \
test_number++;\
if ((cond))\
{\
  test_assert(__FILE__, __LINE__, #cond , "not null");\
}

#endif /* INCLUDE_TEST_H */
