/*
 * test3
 *
 * This file has some test codes of sending and reading messages 
 * through IPC between the parent process and the child process.
 *
 * $Id: test3.c,v 1.1.1.1 2004/05/09 03:04:09 tkudou Exp $
 */

#include <sys/types.h>
#include <unistd.h>
#include <stdio.h>
#include <time.h>

#define WITH_IPCIF
#include "ipcif.h"

#define TEST_HOLD_IMP
#include "test.h"

typedef struct _Msg
{
  long cmd_no;
  char* str;
} Msg;

static Msg msgs[] = 
{
  { 1, "cmd 1, serial 1" },
  { 2, "cmd 2, serial 2" },
  { 3, "cmd 3, serial 3" },
  { 1, "cmd 1, serial 4" },
  { 2, "cmd 2, serial 5" },
  { 3, "cmd 3, serial 6" },
  { 4, "cmd 4, serial 7" },
  { 5, "cmd 5, serial 8" },
  { 3, "cmd 3, serial 9" },
  { 7, "cmd 7, serial 10" },
  { 8, "cmd 8, serial 11" },
  { 0, 0 },
};

const int num_of_try = 100;

void
test1()
{
  IPCIF_Bool ret;
  int n;
  Msg* x;
  
  for (n = 0; n < num_of_try; n++)
  {
    printf(".");
    fflush(stdout);
    for (x = msgs; x->str != 0; x++)
    {
      IPCIF_Msg* msg;
      /* send message */
      msg = ipcif_initMsg(x->cmd_no, IPCIF_S_Unknown, strlen(x->str) + 1, x->str);
      SHOULD_NOT_BE_NULL(msg);
      ret = ipcif_sendMsg(msg);
      SHOULD_BE_TRUE(ret);
      ipcif_deleteMsg(msg);
    }
  }
  printf("\n");
  fflush(stdout);
}

void
test2()
{
  IPCIF_Bool ret;
  int n;
  Msg* x;
  for (n = 0; n < num_of_try; n++)
  {
    printf("*");
    fflush(stdout);
    for (x = msgs; x->str != 0; x++)
    {
      IPCIF_Msg* msg2;
      /* read message */
      while ((msg2 = ipcif_readMsg()) == 0)
      {
	;
      }
      SHOULD_BE_TRUE(x->cmd_no == msg2->cmd_no);
      SHOULD_BE_TRUE(msg2->sender == 0);
      SHOULD_BE_TRUE((strlen(x->str) + 1) == msg2->cmd_data_length);
      SHOULD_BE_TRUE(!strcmp(x->str, msg2->cmd_data));
      ipcif_deleteMsg(msg2);
    }
  }
  printf("\n");
  fflush(stdout);
}

int
main()
{
  pid_t pid;
  struct timespec t;
  
  test_initialize();
  pid = fork();
  if (pid == -1)
  {
    SHOULD_BE_TRUE(0);
    test_show_result();
    return test_get_assert_number();
  }
  
  if (pid)
  {
    ipcif_initialize();
    test1();
    ipcif_finalize(IPCIF_False);
  }
  else
  {
    ipcif_initialize();
    test2();
    ipcif_finalize(IPCIF_True);
  }

  /* sleep 1 second */
  t.tv_sec = 1;
  t.tv_nsec = 0;
  nanosleep(&t, NULL);

  test_show_result();
  return test_get_assert_number();
}

