#include "syousai.h"

#include <ctype.h>

static char ctable[] = {
#include "ctbl.h"
};

static char * change_state[] = {
  " դᤤ ",
  " Ϥ󤫤 ⤸",
  " Ϥ󤫤 ",                    // 1
  " Ϥ󤫤 ʤ",
  " Ϥ󤫤 ",
  " Ϥ󤫤 ",                    // 1
  " 󤫤 ",			 // 2
  " 󤫤 Ҥ餬ʤ",
  " 󤫤 ʤ",
  " 󤫤 ⤸",
  " 󤫤 ",			 // 2
  " 󤫤 ",			 // 2
  " 󤫤 ",
};

SyousaiYomi::SyousaiYomi()
{
  s_c_type = CharType_KANJI;
}

SyousaiYomi::~SyousaiYomi()
{
}

CharType SyousaiYomi::getCtype(int h, int l)
{
  if (h<0x80)
  {	// ANK
    if (isalpha(h))
    {
      if (isupper(h))
      {
	return CharType_ANK_UPCASE_ALPHA;
      }
      else
      {
	return CharType_ANK_DOWNCASE_ALPHA;
      }
    }
    else if (isalnum(h))
    {
      return CharType_ANK_NUM;
    }
//    return CharType_ANK_SYM;
    return CharType_ANK_DOWNCASE_ALPHA;
  }
  CharType t = (CharType) (ctable[(h<<8) + l-0x8000]);
  if (t == CharType_ZEN_SYM || t == CharType_ZEN_DOWNCASE_ALPHA)
  {
    t = CharType_KANJI;
  }
  return t;
}

void SyousaiYomi::modeChange(CharType new_type)
{
  out += change_state[new_type];
}

void SyousaiYomi::convertKanaToHira(EucString * src)
{
  int rp = 0;
  
  int h;
  int l;
  while (0 != (h = (unsigned char) (*src)[rp]))
  {
    l = (unsigned char) (*src)[rp+1];
    if (h >= 0x80)
    {
      if (getCtype(h,l) == CharType_ZEN_KANA)
      {
	(*src)[rp] = 0xa4;
      }
      rp+=2;
    }
    else
    {
      rp++;
    }
  }
}

const EucString * SyousaiYomi::yomiconv(const EucString * src)
{
  int rp = 0;
  int h;
  int l;
  out = "";
  CharType char_type_old = CharType_KANJI;
  CharType char_type_new = CharType_KANJI;
  while (0 != (h = (unsigned char) (*src)[rp]))
  {
    l = (unsigned char) (*src)[rp+1];
    if (l == 0)
    {	// ȾѤȤư
      char_type_new = getCtype(h, 0);
      rp++;
    }
    else if (h < 0x80)
    {
      char_type_new = getCtype(h,0);
      rp++;
    }
    else
    {
      char_type_new = getCtype(h,l);
      rp += 2;
    }
    if (char_type_new != char_type_old)
    {
      modeChange(char_type_new);
      char_type_old =  char_type_new;
    }
    if (char_type_new == CharType_KANJI)
    {
      char buf[3];
      buf[0] = h;
      buf[1] = l;
      buf[2] = 0;
      EucString tmp_str(buf);
      EucString * yomi = KanjiSyousaiYomi::yomiconv(&tmp_str);
      if (yomi)
      {
	convertKanaToHira(yomi);
	out += *yomi;
      }
      else
      {
	out += tmp_str;
      }
      out += ' ';
    }
    else
    {
      if (l == 0)
      {
	out += h;
//	out += ' ';
      }
      else if (h < 0x80)
      {
	out += h;
//	out += ' ';
      }
      else
      {
	out += h;
	out += l;
//	out += ' ';
      }
    }
  }
  return &out;
}

#if 0
void test(SyousaiYomi & syo, char * str)
{
  printf("%s:", str);
  EucString s(str);
  printf("%s\n", syo.yomiconv(&s)->c_str());
}



main()
{
  SyousaiYomi syo;
  
  test(syo,"abcd#0123");
  test(syo,"122");
}
#endif
